/*/../bin/true; exec go run "$0"; */

package main

/*
fillchk.go: scan for repetitions of a single byte (file corruption)
2019-10-24,  wessie & ed <irc.rizon.net>,  MIT-licensed

this impl:
  780 MiB/s with go 1.12.12

see also:
  https://ocv.me/dev/?fillchk.c
  https://ocv.me/dev/?fillchk.cpp
  https://ocv.me/dev/?fillchk.py
  https://ocv.me/dev/?fillchk.pl
  https://ocv.me/dev/?fillchk.go
  https://ocv.me/dev/?fillchk.lua
*/

import (
	"fmt"
	"os"
)

func report(ch byte, p1, p2 int) {
	fmt.Printf("from %08x to %08x, %02x seen 0x%x / %d times\x1b[J\n", p1, p2, ch, p2-p1, p2-p1)
}

func main() {
	var r = os.Stdin
	var p = make([]byte, 1024*16)
	var pp []byte
	var err error
	var n int

	var p0 int
	var ch0 byte
	var pos int = -1
	for {
		n, err = r.Read(p)
		if err != nil {
			break
		}

		pp = p[:n]
		for _, ch := range pp {
			pos++

			if ch == ch0 {
				continue
			}

			if pos-p0 >= 0x400 {
				report(ch0, p0, pos)
			}

			ch0, p0 = ch, pos
		}
	}

	if pos-p0 >= 0x400 {
		report(ch0, p0, pos)
	}

}
